@@ -30,7 +30,7 @@
 # Standard python modules
 #
 #------------------------------------------------------------------------
-import os,string,tempfile,socket
+import os,string,tempfile,socket,datetime
 
 #------------------------------------------------------------------------
 #
@@ -92,6 +92,7 @@ if os.path.exists(tmp_dir + "/Installed"):
 #
 available_file = init_dir + "/Available"
 previous_file  = init_dir + "/Available.prv"
+log_file       = init_dir + "/Log"
 #
 # No longer using the Installed file so remove it. 23 Aug 2003
 #
@@ -169,7 +170,8 @@ def update_available(noreport=False):
               "| tr '\n' ' '" +\
               "| perl -p -e 's|Package: |\n|g; s|Version: ||g'" +\
               "| sort | tail +2 | sed 's| $||' > " + available_file
-    perform.execute(command, noquiet=True) # root is not required.
+    # Use langC in the following since it uses a grep.
+    perform.execute(command, noquiet=True, langC=True) # root is not required.
     os.rename(temporary_file, previous_file)
 
     available_packages = len(open(available_file).readlines())
@@ -189,7 +191,46 @@ def update_available(noreport=False):
         #print "There are " + str(available_packages) + " packages now " +
         #      "available which is " + direction + " the previous count."
         print "This is " + direction + " the previous count",
-        print "with " + newest + " new packages."
+        print "with " + newest + " new",
+        if newest == "1":
+            print "package."
+        else:
+            print "packages."
+
+########################################################################
+#
+# UPDATE LOG
+#
+# Call start_log before any actions, then finish_log when actions are
+# finished. This will write a summary of changes to the log file.
+#
+old_log = tempfile.mktemp()
+def start_log():
+    # Write list of installed to tmp file
+    perform.execute(gen_installed_command_str() + " > " + old_log,
+                    noquiet=1, langC=True)
+
+def finish_log():
+    ts = datetime.datetime.today().isoformat()
+    ts = ts[:-10] # Don't put seconds - it implies too much accuracy
+    # Generate new list of installed and compare to old
+    lf = file(log_file, "a")
+    new_iter = perform.execute(gen_installed_command_str(),
+                               noquiet=1, langC=True, pipe=True)
+    old_iter = file(old_log)
+    for o in old_iter:
+        o = o.strip().split(" ")
+        n = new_iter.next().strip().split(" ")
+        while o[0] <> n[0]:
+            if o[0] < n[0]:
+                lf.write("%s %s %s %s\n" % (ts, "remove", o[0], o[1]))
+                o = old_iter.next().strip().split(" ")
+            elif o[0] > n[0]:
+                lf.write("%s %s %s %s\n" % (ts, "install", n[0], n[1]))
+                n = new_iter.next().strip().split(" ")
+        if o[1] <> n[1]:
+            lf.write("%s %s %s %s\n" % (ts, "upgrade", n[0], n[1]))
+    os.remove(old_log)
 
 #------------------------------------------------------------------------
 #
@@ -209,7 +250,9 @@ def gen_installed_command_str():
 def count_upgrades():
     """Return as a string the number of new upgrades since last update."""
     ifile = tempfile.mktemp()
-    perform.execute(gen_installed_command_str() + " > " + ifile, noquiet=1)
+    # Use langC in the following since it uses a grep.
+    perform.execute(gen_installed_command_str() + " > " + ifile,
+                    langC=True, noquiet=1)
     command = "join " + previous_file + " " + available_file + " |" +\
               "awk '$2 != $3 {print}' | join - " + ifile + " |" +\
               "awk '$4 != $3 {print}' | wc -l | awk '{print $1}' "
@@ -367,7 +410,8 @@ def get_status(pkg):
 
 def get_dependees(pkg):
     """Return a list of other installed pkgs that depend on PKG."""
-    pkginfo = perform.execute("apt-cache rdepends " + pkg, pipe=True)
+    pkginfo = perform.execute("apt-cache rdepends --installed " +
+                              pkg, pipe=True)
     dp = []
     # Watch for changes to apt-cache output.
     if pkginfo.next().strip() <> pkg:
@@ -393,7 +437,8 @@ def get_dependees(pkg):
 
 def get_dependencies(pkg):
     """Return a list of installed packages that PKG depends on."""
-    pkginfo = perform.execute("apt-cache depends " + pkg, pipe=True)
+    pkginfo = perform.execute("apt-cache depends --installed " +
+                              pkg, pipe=True)
     dp = []
     # Watch for changes to apt-cache output.
     if pkginfo.next().strip() <> pkg:
