@@ -537,6 +537,30 @@ def do_install_suggest(packages, type):
               + perform.concat(packages) + suggest_list
     perform.execute(command, root=1)
 
+#-----------------------------------------------------------------------
+#
+# LIST SECTION
+#
+#-----------------------------------------------------------------------
+def do_listsections():
+    command = "apt-cache dumpavail"
+    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+    avail = apt_pkg.ParseTagFile(packages_pipe)
+    sections = []
+    while avail.Step():
+        s = avail.Section.get("Section")
+        if s not in sections:
+            sections.append(s)
+    print "\n".join(sections)
+
+def do_listsection(pattern):
+    command = "apt-cache dumpavail"
+    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+    avail = apt_pkg.ParseTagFile(packages_pipe)
+    while avail.Step():
+        if (pattern == avail.Section.get("Section")):
+            print avail.Section.get("Package")
+
 #------------------------------------------------------------------------
 #
 # LIST INSTALLED
@@ -569,6 +593,10 @@ def do_listnames(pattern, pipe=False):
         command = "apt-cache pkgnames | sort"
     else:
         command = "apt-cache pkgnames | grep -- " + pattern[0] + " | sort"
+    # Start fix for Bug #292581 - pre-run command to check for no output
+    results = perform.execute(command, root=needsudo, pipe=True).readlines()
+    if len(results) == 0: sys.exit(1)
+    # End fix for Bug #292581
     return perform.execute(command, root=needsudo, pipe=pipe)
 
 #------------------------------------------------------------------------
@@ -602,6 +630,7 @@ def map_sources(packages):
     """Return Source package name for each package in PACKAGES.
 
     Note that the source could include a version number.
+    Only replace with source if package exists (Bug#295455).
 """
     Parse = apt_pkg.ParseTagFile(open("/var/lib/dpkg/available","r"))
     sources = []
@@ -612,9 +641,9 @@ def map_sources(packages):
             if Parse.Section.get("Source"):
                 #pkgname = Parse.Section.get("Source").split()[0]
                 pkgname = Parse.Section.get("Source")
-                sources += [pkgname]
+                sources += [[pkgname, Parse.Section.get("Package")]]
             else:
-                sources += [Parse.Section.get("Package")]
+                sources += [[Parse.Section.get("Package"), Parse.Section.get("Package")]]
     return sources
 
 def do_news(packages):
@@ -628,7 +657,8 @@ def do_news(packages):
     ping_host("packages.qa.debian.org")
     results = tempfile.mktemp()
     packages = map_sources(packages)
-    for p in packages:
+    for pk in packages:
+        p = pk[0]
         #
         # Print a header.
         #
@@ -659,7 +689,22 @@ def do_news(packages):
 #
 #------------------------------------------------------------------------
 
-def pool_version(package):
+def pool_version(pkg, orig):
+    #
+    # Check first if package is available, if not then
+    # get version from original. Fixes Bug#295455
+    #
+    revert = False
+    changes.load_dictionaries()
+    available = changes.get_available_list()
+    if pkg not in available:
+        package = orig
+        revert = True
+    else:
+        package = pkg
+    #
+    # Get the version
+    #
     command = "apt-cache show " + package
     packages_pipe = perform.execute(command, noquiet=True, pipe=True)
     Parse = apt_pkg.ParseTagFile(packages_pipe)
@@ -671,6 +716,19 @@ def pool_version(package):
                == 0:
             version = v
             filename = Parse.Section.get("Filename")
+            #
+            # If we reverted to the original package, be sure to get the
+            # changelog from the source package. Fixes Bug#295455
+            #
+            if revert:
+                filename = filename.replace(orig, pkg)
+            #
+            # Special case of local installed packages.
+            # Take a guess... Example is my own locally installed wajig.
+            # 
+            if filename[0:2] == "./":
+                filename = "pool/main/" + filename[2] +\
+                           filename.split("_")[0][1:] + "/" + filename[2:]
     return (filename, version)
 
 def do_changelog(packages):
@@ -685,7 +743,14 @@ def do_changelog(packages):
     ping_host("packages.debian.org")
     results = tempfile.mktemp()
     packages = map_sources(packages)
-    for p in packages:
+    for pk in packages:
+        #
+        # map_sources was changed 17 Feb 2005 to return pairs so that
+        # we can track the original package name in case the source
+        # package is not an installable package. Fixes Bug #295455.
+        #
+        p = pk[0]
+        o = pk[1]
         #
         # Print a header.
         #
@@ -711,7 +776,11 @@ def do_changelog(packages):
         # So move to pointing the to latest version's actual changelog
         # on packages.debian.org. (gjw 30 Dec 2004).
         #
-        (pool, version) = pool_version(pkg)
+        # Extra argument added 17 Feb for fix for Bug#295455. The orig
+        # package name is used to get version if source pkg is not an
+        # installable package.
+        #
+        (pool, version) = pool_version(pkg, o)
         changelog_path = re.sub(r'_[^_]*\.deb$', '/changelog', pool)
         #
         # Generate the command to retireve the changelog.
@@ -774,7 +843,7 @@ def do_removedepend(pkg, purge=False):
     #
     print "Removing:", toremove
     if purge:
-        perform.execute("dpkg --purge " + toremove, root=1)
+        perform.execute("apt-get remove --purge " + toremove, root=1)
     else:
         perform.execute("apt-get remove " + toremove, root=1)
 
@@ -855,8 +924,25 @@ def do_status(packages, snapshot=False):
     # Generate a temporary file of installed packages.
     #
     ifile = tempfile.mktemp()
+    #
+    # Using langC=TRUE here makes it work for other LANG, e.g.,
+    # LANG=ru_RU.koi8r. Seems that the sorting is the key problem. To
+    # test, try:
+    #
+    #   $ wajign status | wc -l
+    #   1762
+    #   $ LANG=ru_RU.koi8r wajign status | wc -l
+    #   1762
+    #
+    # But now set it to False (the default):
+    #
+    #   $ LANG=ru_RU.koi8r wajign status | wc -l
+    #   1449
+    #
+    # See Bug#288852 and Bug#119899.
+    #
     perform.execute(changes.gen_installed_command_str() + " > " + ifile,
-                    noquiet=1)
+                    noquiet=1, langC=True)
     #
     # Build the command to list the status of installed packages.
     #
@@ -991,7 +1077,8 @@ likely that your Packages files are out of date. Doing an UPDATE is optional
         if not re.match("^(y|Y|yes|Yes|YES)$", doit):
             return
     #
-    # Note that we do a "dselect update" rather than an "apt-get update" since the
+    # Note that we do a "dselect update" rather than an
+    # "apt-get update" since the
     # latter does not update dpkg's idea of what's available!
     #
     # if perform.execute("dselect update", root=1) == 0:
@@ -1005,7 +1092,13 @@ likely that your Packages files are out of date. Doing an UPDATE is optional
     # by apt-get so new was not finding the package descriptions.
     #
     if quiet:
-        quietopt = "| egrep -v '(http|ftp)'"
+        #
+        # The quotes carefully chosen below are important. swapping "
+        # and ' when root is running and /bin/sh is used
+        # (rather than sudo) results in a syntax error due to too many
+        # ' being used.
+        #
+        quietopt = '| egrep -v "(http|ftp)"'
     else:
         quietopt = ""
     if perform.execute("dselect update " + quietopt, root=1) == 0:
@@ -1066,14 +1159,20 @@ def do_whatis0(packages):
 #
 #------------------------------------------------------------------------
 def do_whichpkg(filename):
-    """Look for a particular file or pattern in a Debian package.
+    """Search for a particular file or pattern in a Debian package.
+
+    Search the Debian package archive (local installed packages and
+    from the Debian package archive on packages.debian.org) for any
+    packages containing a file with FILENAME in its name. The results
+    are printed to standard output.
 
     Arguments:
     filename    Pattern to find
 
     Returns:"""
 
-    ping_host("packages.debian.org")
+    packages_host = "packages.debian.org"
+    ping_host(packages_host)
     #
     # Print out a suitable heading
     #
@@ -1101,7 +1200,7 @@ def do_whichpkg(filename):
     #
     results = tempfile.mktemp()
     command = "wget --timeout=60 --output-document=" + results +\
-              " http://packages.debian.org/cgi-bin" +\
+              " http://" + packages_host + "/cgi-bin" +\
               "/search_contents.pl\?word=" + filename +\
               "\&searchmode=searchfilesanddirs" +\
               "\&case=insensitive\&version=unstable" +\
@@ -1116,7 +1215,8 @@ def do_whichpkg(filename):
         # Multiple pages of output
         #
         command = "cat " + results + " | " +\
-                  "egrep -v '^<|^$|^Packages search page|^package |//W3C//' | " +\
+                  "egrep -v " +\
+                  "'^ *<|^$|^Packages search page|^package |//W3C//' | " +\
                   "egrep -v 'page=' | " +\
                   "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
         perform.execute(command)
@@ -1134,7 +1234,8 @@ def do_whichpkg(filename):
                   "\&case=insensitive\&searchmode=searchfilesanddirs " +\
                   " 2> /dev/null; " +\
                   "cat " + results + " | " +\
-                  "egrep -v '^<|^$|^Packages search page|^package |//W3C//' | " +\
+                  "egrep -v " +\
+                  "'^ *<|^$|^Packages search page|^package |//W3C//' | " +\
                   "egrep -v 'page=' | " +\
                   "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;" +\
                   "s|^[^<]*>||g;';" +\
@@ -1145,7 +1246,8 @@ def do_whichpkg(filename):
         # A single page of output
         #
         command = "cat " + results + " | " +\
-                  "egrep -v '^<|^$|^Packages search page|^package |//W3C//' | " +\
+                  "egrep -v " +\
+                  "'^ *<|^$|^Packages search page|^package |//W3C//' | " +\
                   "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
         perform.execute(command)
 
