@@ -215,7 +215,7 @@ class JIG(GnomeApp):
         "bug" : ("Bug", ONLYONE, True, "a single named package"),
         "changelog" : ("Change Log", SOMEARGS, False, "a list of packages"),
         "clean" : ("Clean", NOARGS, True, ""),
-        "dependees" : ("Dependees", ONLYONE, False, "a single named package"),
+        "dependents" : ("Dependents", ONLYONE, False,"a single named package"),
         "describe" : ("Describe", SOMEARGS, False, "a list of packages"),
         "detail" : ("Detail", SOMEARGS, False, "a list of packages"),
         "detail_new" : ("Detail New", NOARGS, False, ""),
