@@ -23,11 +23,27 @@
 # Standard python modules
 #
 #------------------------------------------------------------------------
-import os
+import os, getpass
 pause = False
 
 setroot = "/bin/su"
-if os.path.exists("/usr/bin/sudo"):
+#
+# TODO - ONLY SET TO sudo IF USER IS IN THE sudoers FILE. THUS THE
+# FALL BACK IS TO USING su EVEN IF sudo IS INSTALLED.
+#
+user = ""
+
+try:
+    user = os.environ['USER']
+except:
+    pass
+
+try:
+    if not user: user = getpass.getuser()
+except:
+    user = 'unknown'
+
+if os.path.exists("/usr/bin/sudo") and user <> 'root':
     setroot = "/usr/bin/sudo"
     #
     # In case someone is using the non-default install of sudo on
@@ -77,7 +93,7 @@ def concat(args):
 # EXECUTE
 #
 #------------------------------------------------------------------------
-def execute(command, root=0, noquiet=0, display=1, pipe=0):
+def execute(command, root=0, noquiet=0, display=1, pipe=0, langC = False):
     """Ask the operating system to perform a command.
 
     Arguments:
@@ -86,6 +102,7 @@ def execute(command, root=0, noquiet=0, display=1, pipe=0):
     ROOT        If non-zero then root access is required to execute command
     NOQUIET     Suppress the use of quiet (in case command has a redirect)
     PIPE        If True then return a file-like object.
+    LANGC       If LC_TYPE=C is needed (as in join in status command)
 
     Returns:
     
@@ -105,7 +122,7 @@ def execute(command, root=0, noquiet=0, display=1, pipe=0):
             # each part must be sudo.
             command = setroot + " " + command.replace("|", "| %s " % setroot)
         else:
-            if quiet == "" and os.getenv("USER") != "root":
+            if quiet == "" and user != "root":
                 print """
 Using `su' and requiring root password. Install `sudo' to support user
 passwords. See wajig documentation (wajig doc) for details.
@@ -115,11 +132,26 @@ passwords. See wajig documentation (wajig doc) for details.
       command = command
     else:
       command = command + quiet
-    command = "LC_ALL=C; export LC_ALL; set -o noglob; " + command
+    #
+    # Bug#119899 from Michal Politowski <mpol@charybda.icm.edu.pl>
+    # was implemented to ensure handling Polish language okay:
+    #
+    # command = "LC_ALL=C; export LC_ALL; set -o noglob; " + command
+    command = "set -o noglob; " + command
+    #
+    # This worked a long time until Bug#288852 from Serge Matveev
+    # <serge@matveev.spb.ru> reported that locale is not handled -
+    # which is correct and according to the fix of Bug#119899 the
+    # language was specifially put to be C to make the join work in
+    # the status command. So the fix now is to not touch the locale
+    # except if the cammand asks for it through the langC flag.
+    #
+    if langC: command = "LC_ALL=C; export LC_ALL; " + command
     if simulate == 0:
         if pipe == 0:
             #if pause:
             #    command += " | /usr/bin/pager"
+            # return os.system(command)
             return os.system(command)
         else:
             return os.popen(command)
